package simulation;

import commandSystem.BasicInstructionQueue;
import Events.CallFromElevatorEvent;
import Events.CallFromFloorEvent;
import commandSystem.Direction;
import commandSystem.ElevatorListener;
import simulation.Elevator;
import Events.*;


public class Simulation implements ElevatorListener {
	private boolean running = false;
	private int currentFloor = 0;
	private BasicInstructionQueue queue = new BasicInstructionQueue();
	private Elevator elevator;

	public Simulation(Elevator elevator) {
		this.elevator = elevator;
	}

	public void elevatorCall(int floor) {
		System.out.println("elevator call " + floor);
		queue.computeEvent(new CallFromElevatorEvent(floor));
		update();
	}

	public void floorCall(int floor, Direction direction) {
		System.out.println("floor call " + floor + " " + direction);
		queue.computeEvent(new CallFromFloorEvent(floor, direction));
		update();
	}

	public void floorChange(Direction direction) {
		System.out.println("floor change " + direction);
		switch (direction) {
		case UP:
			currentFloor++;
			break;
		case DOWN:
			currentFloor--;
			break;
		}
		queue.computeEvent(new ReachedFloorEvent(currentFloor, direction));
		update();
	}

	private void update() {
		final int next = queue.getNextInstruction();
		System.out.println("next: " + next);
		if (next == -1 || next == currentFloor) {
			System.out.println("stopping");
			elevator.stop();
		}
		else if (next > currentFloor) {
			System.out.println("going up");
			elevator.goUp();
		}
		else if (next < currentFloor) {
			System.out.println("going down");
			elevator.goDown();
		}
	}
}
